import { FC } from 'react';
import { JSXElementConstructor } from 'react';
import { ReactElement } from 'react';
import { ReactNode } from 'react';
import { Ref } from 'react';
import { RefAttributes } from 'react';
import { SymOpts } from '@thi.ng/shader-ast/api/syms';
import { Type } from '@thi.ng/shader-ast';

export declare const Abs: (props: {
    a: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Acos: (props: {
    of: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Add: (props: {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Asin: (props: {
    of: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Assign: (props: {
    to: Ref<ReactNode> | ReactNode;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Atan: (props: {
    y?: number | ReactElement<any, string | JSXElementConstructor<any>> | undefined;
    x?: number | ReactElement<any, string | JSXElementConstructor<any>> | undefined;
    of?: number | ReactElement<any, string | JSXElementConstructor<any>> | undefined;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Bool: (props: {
    value: Bool_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

declare type Bool_2 = boolean;

export declare const Call: (props: {
    fn: string;
    type: Type;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Ceil: (props: {
    value: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Clamp: (props: {
    x: Float_2 | Int_2 | UInt | ReactElement;
    min: Float_2 | Int_2 | UInt | ReactElement;
    max: Float_2 | Int_2 | UInt | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Cos: (props: {
    of: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Cross: (props: {
    a: Vec | ReactElement;
    b: Vec | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Decrement: (props: {
    value: ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Degrees: (props: {
    radians: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Distance: (props: {
    from: Vec | ReactElement;
    to: Vec | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Divide: (props: {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Dot: (props: {
    a: Vec | ReactElement;
    b: Vec | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Exp: (props: {
    a: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Exp2: (props: {
    a: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Float: (props: {
    value: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

declare type Float_2 = number;

export declare const Floor: (props: {
    a: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Fract: (props: {
    of?: number | ReactElement<any, string | JSXElementConstructor<any>> | undefined;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Increment: (props: {
    value: ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Input: (props: {
    type: Type;
    id: string;
    opts?: SymOpts | undefined;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Int: (props: {
    value: Int_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

declare type Int_2 = number;

export declare const InverseSqrt: (props: {
    a: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

declare type ItemType = Int_2 | Float_2 | Bool_2 | ReactElement;

declare type ItemType_2 = Int_2 | Float_2 | Bool_2 | ReactElement;

declare type ItemType_3 = Int_2 | Float_2 | Bool_2 | ReactElement;

export declare const Length: (props: {
    of: Vec | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Log: (props: {
    a: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Log2: (props: {
    a: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Main: FC<MainProps>;

declare interface MainProps {
    children: ReactNode;
}

export declare const Max: (props: {
    a: Float_2 | Int_2 | UInt | ReactElement;
    b: Float_2 | Int_2 | UInt | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Min: (props: {
    a: Float_2 | Int_2 | UInt | ReactElement;
    b: Float_2 | Int_2 | UInt | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Mix: (props: {
    a: Float_2 | ReactElement;
    b: Float_2 | ReactElement;
    c: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Mod: (props: {
    a: Float_2 | ReactElement;
    b: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Mul: (props: {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Normalize: (props: {
    value: Vec | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Output: (props: {
    type: Type;
    id: string;
    opts?: SymOpts | undefined;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Pow: (props: {
    a: Float_2 | ReactElement;
    b: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Radians: (props: {
    degrees: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Return: (props: {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Sign: (props: {
    a: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Sin: (props: {
    of: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const SmoothStep: (props: {
    from: Float_2 | ReactElement;
    to: Float_2 | ReactElement;
    x?: number | ReactElement<any, string | JSXElementConstructor<any>> | undefined;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Sqrt: (props: {
    a: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Step: (props: {
    edge: Float_2 | ReactElement;
    x?: number | ReactElement<any, string | JSXElementConstructor<any>> | undefined;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Sub: (props: {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Sym: (props: {
    type: string;
    id: string;
    opts?: any;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Tan: (props: {
    of: Float_2 | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

declare type UInt = number;

export declare const Uint: (props: {
    value: UInt | ReactElement;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Uniform: (props: {
    type: Type;
    id: string;
    opts?: SymOpts | undefined;
} & {
    children?: ReactNode;
} & RefAttributes<ReactElement<any, string | JSXElementConstructor<any>>>) => ReactNode;

declare type Vec = Vec2_2 | Vec3_2 | Vec4_2;

export declare const Vec2: (props: {
    type?: VecTypeEnum | undefined;
    x?: ItemType | undefined;
    y?: ItemType | undefined;
    value?: Vec2_2 | undefined;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

declare type Vec2_2 = [ItemType, ItemType];

export declare const Vec3: (props: {
    type?: VecTypeEnum | undefined;
    x?: ItemType_2 | undefined;
    y?: ItemType_2 | undefined;
    z?: ItemType_2 | undefined;
    value?: Vec3_2 | undefined;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

declare type Vec3_2 = [ItemType_2, ItemType_2, ItemType_2];

export declare const Vec4: (props: {
    type?: VecTypeEnum | undefined;
    x?: ItemType_3 | undefined;
    y?: ItemType_3 | undefined;
    z?: ItemType_3 | undefined;
    w?: ItemType_3 | undefined;
    value?: Vec4_2 | undefined;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

declare type Vec4_2 = [ItemType_3, ItemType_3, ItemType_3, ItemType_3];

export declare enum VecTypeEnum {
    vec = "vec",
    ivec = "ivec",
    uvec = "uvec",
    bvec = "bvec"
}

export declare const W: (props: {
    of?: ReactElement<any, string | JSXElementConstructor<any>> | ReactElement<any, string | JSXElementConstructor<any>>[] | Vec | undefined;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const X: (props: {
    of?: ReactElement<any, string | JSXElementConstructor<any>> | ReactElement<any, string | JSXElementConstructor<any>>[] | Vec | undefined;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const XY: (props: {
    of?: ReactElement<any, string | JSXElementConstructor<any>> | ReactElement<any, string | JSXElementConstructor<any>>[] | Vec2_2 | Vec3_2 | Vec4_2 | undefined;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const XYZ: (props: {
    of?: ReactElement<any, string | JSXElementConstructor<any>> | ReactElement<any, string | JSXElementConstructor<any>>[] | Vec3_2 | Vec4_2 | undefined;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Y: (props: {
    of?: ReactElement<any, string | JSXElementConstructor<any>> | ReactElement<any, string | JSXElementConstructor<any>>[] | Vec | undefined;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export declare const Z: (props: {
    of?: ReactElement<any, string | JSXElementConstructor<any>> | ReactElement<any, string | JSXElementConstructor<any>>[] | Vec3_2 | Vec4_2 | undefined;
} & {
    children?: ReactNode;
} & RefAttributes<FC>) => ReactNode;

export { }
